"""The entry screen for the game"""

from pymunk import Vec2d
import pyglet

import engine
import levels
import settings
import sound


class LevelScreen(engine.World):
    """The main start screen for the game"""

    def worldCreated(self):
        """The screen was created"""
        #
        # Create the batches
        text = self.addBatch('text')
        #
        back_layer = pyglet.graphics.OrderedGroup(0)
        icon_layer = pyglet.graphics.OrderedGroup(1)
        text_layer = pyglet.graphics.OrderedGroup(2)
        #
        bg = engine.SpriteActor('bg', 'start-bg.jpg', batch=text, group=back_layer)
        bg.position = settings.S.Start.bg_position
        self.addActor(bg)
        #
        # self.tweens.append(
        #     engine.Tween(bg, 'opacity', 255, engine.utils.RandomGenerator(150, 200),
        #                  engine.utils.RandomGenerator(3, 5),
        #                  engine.Tween.sinInOut, repeat=True)
        # )
        #
        logo = engine.SpriteActor('logo', 'logo.png', batch=text, group=text_layer)
        logo.position = settings.S.Start.logo_position
        self.addActor(logo)
        #
        back = engine.SpriteActor('back-button', 'back.png', batch=text, group=text_layer)
        back.position = settings.S.Level.back_btn_position
        back.linkEvent(engine.events.E_LEFT_CLICK, self.backClicked)
        self.addActor(back)
        #
        self.objects = []
        for idx, level in enumerate(levels.levels):
            #
            # The text for the level
            new_level = engine.TextActor(
                level.Level.name, pyglet.text.decode_attributed(
                    '{color (255,255,255,255)}{font_name "Sci Fied 2002"}%s' % level.Level.name), batch=text, group=text_layer)
            new_level.x = settings.S.Level.icons_start_x + (float(idx % settings.S.Level.icons_per_row) /
                                                            (settings.S.Level.icons_per_row - 1) - 0.5) * settings.S.Level.icons_width
            new_level.y = settings.S.Level.icons_start_y + settings.S.Level.icons_rows_height * (idx // settings.S.Level.icons_per_row)
            new_level.linkEvent(engine.events.E_LEFT_CLICK, self.startLevel, idx)
            self.addActor(new_level)
            new_level.target_x = new_level.x
            new_level.target_y = new_level.y
            new_level.level_num = idx
            #
            # The image for the level
            img = engine.SpriteActor('icon-%d' % idx, level.Level.icon, batch=text, group=icon_layer)
            img.x = img.target_x = new_level.x
            img.y = img.target_y = new_level.y - settings.S.Level.icon_offset_y
            self.addActor(img)
            img.linkEvent(engine.events.E_LEFT_CLICK, self.startLevel, idx)
            img.level_num = idx
            #
            self.objects.append(new_level)
            self.objects.append(img)
        #
        self.current_level = settings.S.start_level
        self.level_cache = {}
        #

    def applyTweens(self, start_offset_x, start_offset_y, end_offset_x, end_offset_y, after=None, skip=None):
        """Apply the tweens to bring items on the screen"""
        #
        # Tween in the image
        idx = 0
        for item in self.objects:
            if item.level_num != skip:
                self.tweens.append(
                    engine.Tween(
                        item, 'position', Vec2d(item.target_x + start_offset_x, item.target_y + start_offset_y),
                        Vec2d(item.target_x + end_offset_x, item.target_y + end_offset_y),
                        settings.S.Level.tween_in_duration, engine.Tween.sinOut,
                        delay=float(settings.S.Level.tween_in_delay),
                        after=after if idx == 0 else None
                    )
                )
                idx += 1

    def onKeyPress(self, symbol, modifier):
        """Handle key presses"""
        super(LevelScreen, self).onKeyPress(symbol, modifier)
        #
        if symbol == pyglet.window.key.ESCAPE:
            self.engine.setCurrentWorld('start-screen')
            return pyglet.event.EVENT_HANDLED

    def onKeyRelease(self, symbol, modifier):
        """Handle key presses"""
        super(LevelScreen, self).onKeyRelease(symbol, modifier)
        #
        if symbol == pyglet.window.key.ESCAPE:
            return pyglet.event.EVENT_HANDLED
        #
        # Debugging code
        if self.options.cheat:
            if symbol == pyglet.window.key.P:
                self.log.debug('Trying to start sinister music')
                sound.Music.sinister.pause()

    def startClicked(self, obj, arg):
        """The start button was clicked"""
        self.log.info('Start was clicked - current level is %d' % self.current_level)
        #
        # Find the level
        if self.current_level in self.level_cache:
            level = self.level_cache[self.current_level]
        else:
            level_module = levels.levels[self.current_level]
            level = self.level_cache[self.current_level] = level_module.Level(self.options)
            self.engine.addWorld(level)
        #
        # Switch to it
        self.engine.fadeToWorld(level.name, settings.S.fade_screen_delay)
        #
        # Move on to next level
        self.current_level += 1
        if self.current_level >= len(levels.levels):
            self.current_level = 0

    def startLevel(self, obj, level):
        """Start a particular level"""
        sound.Sounds.click_accept.play()
        self.current_level = level
        self.applyTweens(
            0, 0, 0, settings.S.Level.tween_out_offset_y, after=self._startLevelReady,
            skip=level
        )

    def _startLevelReady(self):
        """Ready to start the level"""
        self.startClicked(None, None)

    def backClicked(self, obj, arg):
        """Back was clicked"""
        self.log.info('Back was clicked')
        sound.Sounds.click_backward.play()
        self.applyTweens(0, 0, settings.S.Level.tween_in_offset_x, 0, after=self._backDone)

    def _backDone(self):
        """Back is ready to go"""
        self.engine.setCurrentWorld('start-screen')

    def activateWorld(self):
        """The world was activated"""
        super(LevelScreen, self).activateWorld()
        #
        if not sound.Music.sinister.playing:
            self.log.info('Trying to restart the music')
            sound.Music.sinister.play()
        self.applyTweens(settings.S.Level.tween_in_offset_x, 0, 0, 0)

    def deactivateWorld(self, next_world_name):
        """Deactivate the world"""
        if next_world_name != 'start-screen':
            sound.Music.sinister.pause()